/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JOTMComponent.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.component.jotm;

import java.rmi.RemoteException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.SystemException;
import javax.transaction.TransactionManager;

import org.objectweb.jotm.Current;
import org.objectweb.jotm.TimerManager;
import org.objectweb.jotm.TransactionFactory;
import org.objectweb.jotm.TransactionFactoryImpl;
import org.ow2.easybeans.component.api.EZBComponentException;
import org.ow2.easybeans.component.itf.TMComponent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Creates and binds the transaction factory and usertransaction object.
 * @author Florent Benoit
 */
public class JOTMComponent implements TMComponent {

    /**
     * Default Transaction timeout.
     */
    private static final int DEFAULT_TIMEOUT = 60;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(JOTMComponent.class);

    /**
     * Transaction manager reference.
     */
    private TransactionManager transactionManager = null;

    /**
     * Transaction timeout (in seconds).
     */
    private int timeout = DEFAULT_TIMEOUT;

    /**
     * Init method.<br/> This method is called before the start method.
     * @throws EZBComponentException if the initialization has failed.
     */
    public void init() throws EZBComponentException {

    }

    /**
     * Start method.<br/> This method is called after the init method.
     * @throws EZBComponentException if the start has failed.
     */
    public void start() throws EZBComponentException {

        // Creates a Transaction factory
        TransactionFactory transactionFactory;
        try {
            transactionFactory = new TransactionFactoryImpl();
        } catch (RemoteException e) {
            throw new EZBComponentException("Cannot create transaction factory", e);
        }

        // Bind it
        try {
            new InitialContext().rebind("TMFactory", transactionFactory);
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot bind transaction factory", e);
        }

        transactionManager = new Current(transactionFactory);
        try {
            transactionManager.setTransactionTimeout(this.timeout);
        } catch (SystemException se) {
            throw new EZBComponentException("Cannot set Transaction Timeout", se);
        }
        try {
            new InitialContext().rebind(JNDI_NAME, transactionManager);
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot bind user transaction", e);
        }

        // info
        logger.info("Register {0} as transaction manager object", JNDI_NAME);
    }

    /**
     * Stop method.<br/> This method is called when component needs to be
     * stopped.
     * @throws EZBComponentException if the stop is failing.
     */
    public void stop() throws EZBComponentException {
        // Unbind factory
        try {
            new InitialContext().unbind("TMFactory");
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot unbind transaction factory", e);
        }

        // Unbind user transaction object
        try {
            new InitialContext().unbind(JNDI_NAME);
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot unbind user transaction", e);
        }

        // Stop timer
        TimerManager.stop(true);
    }

    /**
     * Gets the transaction manager object.
     * @return instance of the transaction manager
     */
    public TransactionManager getTransactionManager() {
        return transactionManager;
    }

    /**
     * Set the Transaction Timeout.
     * @param timeout Timeout (in seconds)
     */
    public void setTimeout(final int timeout) {
        this.timeout = timeout;
    }
}
