/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Deployment.java 2501 2008-02-26 11:02:03Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.ow2.easybeans.deployment.annotations.analyzer.configurator.metadata.ejbjar.EjbJarArchiveMetadataConfigurator;
import org.ow2.easybeans.deployment.annotations.exceptions.ResolverException;
import org.ow2.easybeans.deployment.annotations.helper.ExtraMetadataHelper;
import org.ow2.easybeans.deployment.annotations.helper.ResolverHelper;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarArchiveMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarClassMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.LibrariesAnnotationMetadata;
import org.ow2.easybeans.deployment.xml.EJB3DeploymentDesc;
import org.ow2.easybeans.deployment.xml.EJB3DeploymentDescException;
import org.ow2.easybeans.deployment.xml.EasyBeansDeploymentDesc;
import org.ow2.easybeans.deployment.xml.helper.ExtractUsefullClasses;
import org.ow2.easybeans.deployment.xml.helper.MetadataMerge;
import org.ow2.easybeans.deployment.xml.helper.MetadataSpecificMerge;
import org.ow2.easybeans.deployment.xml.specific.EasyBeansDD;
import org.ow2.easybeans.deployment.xml.struct.EJB3;
import org.ow2.util.deployment.annotations.analyzer.AnalyzerException;
import org.ow2.util.deployment.annotations.analyzer.ArchiveScanner;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class will parse given ejb-jar file and completes metadata by using
 * resolver.
 * @author Florent Benoit
 */
public class Deployment {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Deployment.class);

    /**
     * Archive which will be analyzed.
     */
    private IArchive archive = null;

    /**
     * Extra archives.
     */
    private List<IArchive> extraArchives = null;

    /**
     * Configurator.
     */
    private EjbJarArchiveMetadataConfigurator ejbJarArchiveMetadataConfigurator;

    /**
     * Metadata of configurator.
     */
    private EjbJarArchiveMetadata ejbJarArchiveMetadata;

    /**
     * Constructor.<br> Archive which will be used when analyzing.
     * @param archive the archive to analyze.
     */
    public Deployment(final IArchive archive) {
        this.archive = archive;
        reset();
    }

    /**
     * Build a new Annotation Deployment Analyzer.
     */
    public void reset() {
        this.ejbJarArchiveMetadataConfigurator = new EjbJarArchiveMetadataConfigurator();
        this.ejbJarArchiveMetadata = ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata();
    }

    /**
     * Analyzes the jarFile.
     * @throws AnalyzerException if analyze of jar file fails.
     * @throws ResolverException if resolver fails.
     * @throws EJB3DeploymentDescException if parsing fails.
     */
    @SuppressWarnings("boxing")
    public void analyze() throws AnalyzerException, EJB3DeploymentDescException, ResolverException {

        ArchiveScanner archiveScanner = new ArchiveScanner(archive);
        archiveScanner.addArchiveConfigurator(ejbJarArchiveMetadataConfigurator);

        // Get EJB3 DD.
        EJB3 ejb3 = EJB3DeploymentDesc.getEjb3(archive);

        // Get Specific DD
        EasyBeansDD easyBeansDD = EasyBeansDeploymentDesc.getEasyBeansDD(archive);

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();

        // if metadata is complete then filter class to scan
        if (ejb3 != null && ejb3.isMetadataComplete()) {
            // disable annotation parsing
            ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(true);

            // get class to scan from deployment descriptor
            List<String> nextClassNameToScan = ExtractUsefullClasses.extract(ejb3);

            // first scan
            List<EjbJarClassMetadata> metadataWithDirectDependencies = new ArrayList<EjbJarClassMetadata>();
            Collection<EjbJarClassMetadata> ejbJarClassMetadataCollection = ejbJarArchiveMetadata
                    .getEjbJarClassMetadataCollection();
            archiveScanner.scanCollection(nextClassNameToScan);

            // first iteration : beans + interface + bean super class
            // after : interface + bean super class
            while (nextClassNameToScan.size() != 0) {
                nextClassNameToScan.clear();
                // try to add new class from new metadata
                for (EjbJarClassMetadata ejbJarClassMetadata : ejbJarClassMetadataCollection) {
                    if (metadataWithDirectDependencies.contains(ejbJarClassMetadata)) {
                        continue;
                    }
                    // add direct dependencies
                    nextClassNameToScan.add(ejbJarClassMetadata.getSuperName());
                    nextClassNameToScan.addAll(Arrays.asList(ejbJarClassMetadata.getInterfaces()));
                    // specify to ignore this metadata in future
                    metadataWithDirectDependencies.add(ejbJarClassMetadata);
                }
                // continue the scan
                archiveScanner.scanCollection(nextClassNameToScan);
            }

            // scan finished
            archiveScanner.endScan();
        } else {
            // enable annotation parsing because metadata descriptor is not complete
            ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(false);
            archiveScanner.scanAll();
        }

        // time if debugging
        if (logger.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            if (logger.isDebugEnabled()) {
                logger.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
            }
        }

        LibrariesAnnotationMetadata librariesAnnotationMetadata = new LibrariesAnnotationMetadata();
        // Extra archives ?
        if (extraArchives != null) {
            List<EjbJarArchiveMetadata> ejbJarAnnotationMetadataList = new ArrayList<EjbJarArchiveMetadata>();

            // Analyze the extra archives (with annotation parsing and blind scan)
            for (IArchive extraArchive : extraArchives) {
                ArchiveScanner extraArchiveScanner = new ArchiveScanner(extraArchive);
                EjbJarArchiveMetadataConfigurator ejbJarExtraArchiveMetadataConfigurator =
                    new EjbJarArchiveMetadataConfigurator();
                extraArchiveScanner.addArchiveConfigurator(ejbJarExtraArchiveMetadataConfigurator);
                extraArchiveScanner.scanAll();
                ejbJarAnnotationMetadataList.add(ejbJarExtraArchiveMetadataConfigurator.getEjbJarArchiveMetadata());
            }
            librariesAnnotationMetadata.setEjbJarAnnotationMetadataList(ejbJarAnnotationMetadataList);
        }

        // Extra metadata ? complete them
        ExtraMetadataHelper.complete(ejbJarArchiveMetadata, librariesAnnotationMetadata);

        // prepare the metadata for merge
        ejbJarArchiveMetadata.setEjb3(ejb3);

        // Merge DD with annotations metadata
        MetadataMerge.merge(ejbJarArchiveMetadata);

        // Merge specific DD
        ejbJarArchiveMetadata.setEasyBeansDD(easyBeansDD);
        MetadataSpecificMerge.merge(ejbJarArchiveMetadata);

        // Complete metadata
        long tResolverStart = System.currentTimeMillis();
        ResolverHelper.resolve(ejbJarArchiveMetadata);
        // time if debugging
        if (logger.isDebugEnabled()) {
            long tResolverEnd = System.currentTimeMillis();
            if (logger.isDebugEnabled()) {
                logger.debug("Resolver on metadata from {0} took {1} ms.'", archive.getName(),  (tResolverEnd - tResolverStart));
            }
        }

        if (logger.isDebugEnabled()) {
            for (EjbJarClassMetadata classAnnotationMetadata : ejbJarArchiveMetadata.getEjbJarClassMetadataCollection()) {
                logger.debug("Result for class = " + classAnnotationMetadata);
            }
        }
    }

    /**
     * @return the archive metadata
     */
    public EjbJarArchiveMetadata getEjbJarArchiveMetadata() {
        return ejbJarArchiveMetadata;
    }

    /**
     * Add extra archives for finding classes.
     * @param extraArchives the given archives.
     */
    public void setExtraArchives(final List<IArchive> extraArchives) {
        this.extraArchives = extraArchives;
    }

    /**
     * @return the archive of this deployment.
     */
    public IArchive getArchive() {
        return archive;
    }
}
