/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JNDIResolver.java 2447 2008-02-14 17:09:49Z gaellalire $
 * --------------------------------------------------------------------------
 */
package org.ow2.easybeans.deployment.resolver;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.easybeans.container.JContainer3;
import org.ow2.easybeans.deployment.Deployment;
import org.ow2.easybeans.deployment.annotations.impl.JCommonBean;
import org.ow2.easybeans.deployment.annotations.impl.JLocal;
import org.ow2.easybeans.deployment.annotations.impl.JRemote;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarArchiveMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarClassMetadata;
import org.ow2.easybeans.util.url.URLUtils;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Classes used to resolve JNDI Name for a given deployment.
 * @author Florent Benoit
 */
public class JNDIResolver {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(JNDIResolver.class);

    /**
     * Name.
     */
    public static final String NAME = "jndi.resolver";

    /**
     * Map with key = interface name, value = jndi name.
     */
    private Map<String, String> interfaces = null;

    /**
     * Map with key = bean name, value = &lt;interface name, jndi name&gt;.
     */
    private Map<String, Map<String, String>> beans = null;

    /**
     * Map with key = the ejb-jar name and as value : For a given ejb-name there
     * is a map with bean name and between interface/jndi name.
     */
    private Map<String, Map<String, Map<String, String>>> jndiNameForEjbInJarFile = null;

    /**
     * Constructor.
     */
    public JNDIResolver() {
        interfaces = new HashMap<String, String>();
        beans = new HashMap<String, Map<String, String>>();
        jndiNameForEjbInJarFile = new HashMap<String, Map<String, Map<String, String>>>();
    }

    /**
     * Build a new resolver with the given deployment.
     * @param deployment given deployment object to use.
     */
    public JNDIResolver(final Deployment deployment) {
        this();
        addDeployment(deployment);
    }

    /**
     * Add a given deployment object to this resolver.
     * @param deployment to add to the resolver.
     */
    public void addDeployment(final Deployment deployment) {

        // Get metadata for jar file analyzed
        EjbJarArchiveMetadata ejbJarAnnotationMetadata = deployment.getEjbJarArchiveMetadata();

        // Get Filename for file:// URL
        IArchive archive = deployment.getArchive();
        String name = archive.getName();
        URL url;
        try {
            url = archive.getURL();
        } catch (ArchiveException e) {
            throw new IllegalStateException("Cannot get the URL on the archive '" + archive + "'.", e);
        }

        //TODO: How to handle ejb-link for OSGi bundle (url is not using file://)
        if ("file".equals(url.getProtocol())) {
            File f = null;
            try {
                f = URLUtils.urlToFile(archive.getURL());
            } catch (ArchiveException e) {
                throw new IllegalStateException("Cannot get URL of the archive '" + archive + "'", e);
            }
            name = f.getName();
        }
        addEjbJarAnnotationMetadata(ejbJarAnnotationMetadata, name);
    }

    /**
     * Adds the given metadata to the resolver.
     * @param ejbJarAnnotationMetadata the metadata for a given jar file
     * @param ejbJarFileName the name of the jar file which contains the ejbs.
     */
    public void addEjbJarAnnotationMetadata(final EjbJarArchiveMetadata ejbJarAnnotationMetadata,
            final String ejbJarFileName) {
        // For each bean, get the interfaces and add the jndiName mapping.
        for (EjbJarClassMetadata classAnnotationMetadata : ejbJarAnnotationMetadata.getEjbJarClassMetadataCollection()) {
            if (classAnnotationMetadata.isBean()) {

                // Also, go through the EJB annotations (using ejb name)
                JCommonBean jCommonBean = classAnnotationMetadata.getJCommonBean();
                String mappedName = null;
                String beanName = null;
                List<String> aliases = null;
                if (jCommonBean != null) {
                    // jndi name ?
                    mappedName = jCommonBean.getMappedName();
                    // bean name ?
                    beanName = jCommonBean.getName();

                    // Aliases ?
                    aliases = jCommonBean.getAliases();

                }

                // Add Local and Remote interfaces
                JLocal localItfs = classAnnotationMetadata.getLocalInterfaces();
                JRemote remoteItfs = classAnnotationMetadata.getRemoteInterfaces();
                if (localItfs != null) {
                    for (String itf : localItfs.getInterfaces()) {
                        addInterface(itf, classAnnotationMetadata.getClassName(), "Local", mappedName, beanName, aliases,
                                ejbJarFileName);
                    }
                }
                if (remoteItfs != null) {
                    for (String itf : remoteItfs.getInterfaces()) {
                        addInterface(itf, classAnnotationMetadata.getClassName(), "Remote", mappedName, beanName, aliases,
                                ejbJarFileName);
                    }
                }

                // Remote Home
                String remoteHome = classAnnotationMetadata.getRemoteHome();
                if (remoteHome != null) {
                    addInterface(remoteHome, classAnnotationMetadata.getClassName(), null, null, beanName, null, ejbJarFileName);
                }
                // Local Home
                String localHome = classAnnotationMetadata.getLocalHome();
                if (localHome != null) {
                    addInterface(localHome, classAnnotationMetadata.getClassName(), null, null, beanName, null, ejbJarFileName);
                }
            }
        }
    }

    /**
     * Add the jndi name for a given interface and a given bean name (may be
     * null).
     * @param itf the interface of the bean
     * @param beanClassName the class of the bean
     * @param mode local/remote
     * @param mappedName the mappedName (could be used as JNDI name)
     * @param beanName the name of the bean
     * @param aliases the list of the alias for the ejb
     * @param ejbJarFileName the name of the jar file which contains the ejbs.
     */
    private void addInterface(final String itf, final String beanClassName, final String mode, final String mappedName,
            final String beanName, final List<String> aliases, final String ejbJarFileName) {
        String jndiName = JContainer3.jndiNameEncode(beanClassName, itf, mode);
        if (mappedName != null) {
            jndiName = mappedName;
        }
        String itfName = itf.replace("/", ".");
        interfaces.put(itfName, jndiName);

        // For each bean and its aliases
        List<String> ejbNames = new ArrayList<String>();
        if (beanName != null && !beanName.equals("")) {
            ejbNames.add(beanName);
        }
        if (aliases != null) {
            for (String alias : aliases) {
                ejbNames.add(alias);
            }
        }

        for (String ejbName : ejbNames) {
            Map<String, String> tmpBeanMap = beans.get(ejbName);
            if (tmpBeanMap == null) {
                tmpBeanMap = new HashMap<String, String>();
                beans.put(ejbName, tmpBeanMap);
            }
            tmpBeanMap.put(itfName, jndiName);
            beans.put(ejbName, tmpBeanMap);

            logger.debug("Adding jndi name {0} for beanName {1} with itf {2} and className {3}", jndiName, ejbName, itfName,
                    beanClassName);

            // And now, for the given ejbjar
            Map<String, Map<String, String>> jndiNameForBeans = jndiNameForEjbInJarFile.get(ejbJarFileName);
            if (jndiNameForBeans == null) {
                jndiNameForBeans = new HashMap<String, Map<String, String>>();
                jndiNameForEjbInJarFile.put(ejbJarFileName, jndiNameForBeans);
            }
            // bean is existing ?
            Map<String, String> dataForBean = jndiNameForBeans.get(ejbName);
            if (dataForBean == null) {
                dataForBean = new HashMap<String, String>();
                jndiNameForBeans.put(ejbName, dataForBean);
            }
            dataForBean.put(itfName, jndiName);
        }

        if (ejbNames.size() == 0) {
            logger.debug("Adding jndi name {0} with itf {1} and className {2}", jndiName, itfName, beanClassName);
        }
    }

    /**
     * Gets jndi name for a given interface and a bean name.
     * @param itf the name of the interface.
     * @param beanName the name of the bean.
     * @return the jndi name or null if not found.
     */
    public String getJndiNameInterface(final String itf, final String beanName) {
        // ejb-link with ejb-jar ?
        if (beanName != null && beanName.indexOf("#") > 0) {
            // Extract filename (before #)
            String[] data = beanName.split("#");
            String fName = data[0];
            String ejbName = data[1];

            // Get ejbJar map
            Map<String, Map<String, String>> beans = jndiNameForEjbInJarFile.get(fName);
            Map<String, String> bean = beans.get(ejbName);
            if (bean != null) {
                return bean.get(itf);
            }
            throw new IllegalStateException("No bean with name '" + ejbName + "' was found with ejb-link '" + beanName
                    + "' in jar file '" + jndiNameForEjbInJarFile + "'.");
        }

        // Not ejb-link with #
        if (beanName != null && !beanName.equals("")) {
            Map<String, String> lst = beans.get(beanName);
            if (lst != null) {
                return lst.get(itf);
            }
            throw new IllegalStateException("No bean with name '" + beanName + "' was found.");

        }
        return interfaces.get(itf);

    }

    /**
     * @return string representation of this object
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("JNDIResolver[");
        sb.append("Interfaces =[");
        sb.append(interfaces);
        sb.append("], Beans =");
        sb.append(beans);
        sb.append("]]");

        return sb.toString();
    }
}
