/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TxEntityManagerHandler.java 2101 2007-12-06 15:56:03Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.persistence;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.Transaction;

import org.ow2.easybeans.transaction.JTransactionManager;

/**
 * This class is used to handle different entity manager for Transaction Scoped
 * Persistence Context. It associates one EntityManager to a given transaction
 * @author Florent Benoit
 */
public class TxEntityManagerHandler {

    /**
     * EntityManager factory that will create entity manager.
     */
    private EntityManagerFactory entityManagerFactory = null;

    /**
     * Map between transactions and the entity manager.
     */
    private Map<Transaction, EntityManager> entityManagers = null;

    /**
     * Entity Manager used for a single method call when there is no TX. An
     * interceptor will request to create a new entity manager and will close it
     * at the end of the method.
     */
    private ThreadLocal<Stack<EntityManager>> threadEntityManager = new ThreadLocal<Stack<EntityManager>>();

    /**
     * Build a new handler of these entity managers.
     * @param entityManagerFactory Factory used to create the entity manager (if
     *        no entity manager is already associated to the current
     *        transaction)
     */
    public TxEntityManagerHandler(final EntityManagerFactory entityManagerFactory) {
        this.entityManagerFactory = entityManagerFactory;
        this.entityManagers = new HashMap<Transaction, EntityManager>();
    }

    /**
     * Gets the current entity manager (or create one) for the given transaction
     * if there is one.
     * @return an entitymanager
     */
    public synchronized EntityManager getCurrent() {
        EntityManager current = null;

        // Get current transaction (if any)
        Transaction currentTx = null;
        try {
            currentTx = JTransactionManager.getTransactionManager().getTransaction();
        } catch (SystemException e) {
            throw new IllegalStateException("Cannot get current transaction", e);
        }

        // get status of the current tx (if any)
        int statusTx = Status.STATUS_UNKNOWN;
        if (currentTx != null) {
            try {
                statusTx = currentTx.getStatus();
            } catch (SystemException e) {
                throw new IllegalStateException("Cannot get the status on the current transaction", e);
            }
        }
        /**
         * If the entity manager is called when no JTA transaction is in
         * progress, a persistence context is created and destroyed to service
         * the method call only, and any entities loaded from the database will
         * immediately become detached at the end of the method call.
         */
        if (currentTx == null || !(statusTx == Status.STATUS_ACTIVE)) {
            Stack<EntityManager> stack = threadEntityManager.get();
            if (stack == null) {
                throw new IllegalStateException("No EntityManager stack associated on the current thread without TX."
                        + "NoTxInterceptor may not have been called");
            }
            return stack.peek();
        }

        current = entityManagers.get(currentTx);
        /**
         * If the entity manager is called and there is an existing persistence
         * context bound to the current JTA transaction, the call takes place in
         * that context.
         */
        if (current == null) {
            /**
             * If the entity manager is called and there is no persistence
             * context associated with the current JTA transaction, a new
             * persistence context will be created and bound to the JTA
             * transaction, and the call will take place in that context.
             */
            current = buildNewTxEntityManager(currentTx);
        }

        return current;
    }

    /**
     * Builds a new Entity manager for the given transaction.
     * @param tx transaction on which associate the entity manager
     * @return built entity manager.
     */
    private EntityManager buildNewTxEntityManager(final Transaction tx) {
        EntityManager entityManager = entityManagerFactory.createEntityManager();

        /**
         * The persistence context is created and then associated with the
         * current JTA transaction. The persistence context ends when the
         * associated JTA transaction commits or rolls back,
         */
        try {
            tx.registerSynchronization(new TxEntityManagerLifeCycle(entityManager, tx, this));
        } catch (IllegalStateException e) {
            throw new IllegalStateException("Cannot register Entity manager lifecycle", e);
        } catch (RollbackException e) {
            throw new IllegalStateException("Cannot register Entity manager lifecycle", e);
        } catch (SystemException e) {
            throw new IllegalStateException("Cannot register Entity manager lifecycle", e);
        }

        // add in the managed entitymanager
        entityManagers.put(tx, entityManager);

        return entityManager;
    }

    /**
     * Builds a new Entity manager used when there is no transaction.
     * @return built entity manager.
     */
    private EntityManager buildNoTxEntityManager() {
        return entityManagerFactory.createEntityManager();
    }

    /**
     * Release the entity manager associated to the given tx.
     * @param tx tx to removed
     */
    public void release(final Transaction tx) {
        entityManagers.remove(tx);
    }

    /**
     * Sets the current entity manager (used when to transaction is active).
     */
    public void addCurrent() {
        Stack<EntityManager> stackEntityManager = threadEntityManager.get();
        // No stack yet, create a new one
        if (stackEntityManager == null) {
            stackEntityManager = new Stack<EntityManager>();
            threadEntityManager.set(stackEntityManager);
        }
        // Add a new entity manager on the stack
        stackEntityManager.push(buildNoTxEntityManager());
    }

    /**
     * Sets back to the previous entity manager and close the current entity
     * manager.
     */
    public void closeCurrentAndReturnToPrevious() {
        Stack<EntityManager> stack = threadEntityManager.get();

        // Get entitymanager on the current stack
        EntityManager current = stack.peek();

        // Remove the element in the stack
        stack.pop();

        // If stack is now empty, reset it
        if (stack.empty()) {
            threadEntityManager.set(null);
        }

        // close the current
        if (current != null) {
            current.close();
        }

    }
}
