/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:PersistenceXmlFileAnalyzer.java 1537 2007-07-08 15:31:22Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.persistence.xml;

import java.net.URL;

import javax.persistence.spi.PersistenceProvider;

import org.ow2.easybeans.persistence.PersistenceUnitManager;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Class used to analyze a given archive (by analyzing the persistence.xml file
 * if any).
 * @author Florent Benoit
 */
public final class PersistenceXmlFileAnalyzer {

    /**
     * Directory where persistence.xml file should be.
     */
    private static final String DIRECTORY_PERSISTENCE_XML_FILE = "META-INF";

    /**
     * Name of the persistence.xml file.
     */
    private static final String PERSISTENCE_XML_FILE = "persistence.xml";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(PersistenceXmlFileAnalyzer.class);

    /**
     * Utility class, no public constructor.
     */
    private PersistenceXmlFileAnalyzer() {

    }

    /**
     * Detects and analyze the META-INF/persistence.xml file.
     * @param archive the file to analyze (or directory) in order to find a
     *        persistence.xml file.
     * @param classLoader the classloader used to load the persistence provider
     *        class.
     * @return a persistence unit manager (which can manage the peristence
     *         contexts).
     * @throws PersistenceXmlFileAnalyzerException if detection or analyze
     *         fails.
     */
    public static PersistenceUnitManager analyzePersistenceXmlFile(final IArchive archive, final ClassLoader classLoader)
            throws PersistenceXmlFileAnalyzerException {
        URL persistenceXmlURL = null;
        try {
            persistenceXmlURL = archive.getResource(DIRECTORY_PERSISTENCE_XML_FILE + '/' + PERSISTENCE_XML_FILE);
        } catch (ArchiveException e) {
            throw new PersistenceXmlFileAnalyzerException("Cannot check if entry '" + DIRECTORY_PERSISTENCE_XML_FILE
                    + '/' + PERSISTENCE_XML_FILE + "' is present on the file '" + archive.getName() + "'.", e);
        }

        // Now, do the parsing and fill the structure.
        JPersistenceUnitInfo[] persistenceUnitInfos = null;
        if (persistenceXmlURL != null) {
            try {
                persistenceUnitInfos = JPersistenceUnitInfoHelper.getPersistenceUnitInfo(persistenceXmlURL);
            } catch (JPersistenceUnitInfoException e) {
                throw new PersistenceXmlFileAnalyzerException("Cannot parse the URL '" + persistenceXmlURL + "'.", e);
            }

            for (JPersistenceUnitInfo persistenceUnitInfo : persistenceUnitInfos) {
                try {
                    // sets the archive
                    persistenceUnitInfo.addJarFile(archive.getURL());

                    // Set the root url
                    persistenceUnitInfo.setPersistenceUnitRootUrl(archive.getURL());
                } catch (ArchiveException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot get the URL on the jar file '" + archive.getName()
                            + "'.", e);
                }

                // sets the classloader
                persistenceUnitInfo.setClassLoader(classLoader);

                if (persistenceUnitInfo.getPersistenceProviderClassName() == null) {
                    throw new PersistenceXmlFileAnalyzerException("No Persistence provider has been set");
                }

                // instantiate persistence provider
                Class persistenceProviderClass;
                try {
                    ClassLoader ctxLoader = Thread.currentThread().getContextClassLoader();
                    persistenceProviderClass = ctxLoader.loadClass(persistenceUnitInfo
                            .getPersistenceProviderClassName());
                } catch (ClassNotFoundException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot load the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.");
                }
                PersistenceProvider persistenceProvider;
                try {
                    persistenceProvider = (PersistenceProvider) persistenceProviderClass.newInstance();
                } catch (InstantiationException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot instantiate the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.", e);
                } catch (IllegalAccessException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot instantiate the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.", e);
                }

                // Set persistence provider
                persistenceUnitInfo.setPersistenceProvider(persistenceProvider);

            }

            // create persistence unit manager
            return new PersistenceUnitManager(persistenceUnitInfos);
        }
        // nothing found, return nothing(null)
        return null;
    }

}
