/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CommonMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.metadata;

import java.util.Arrays;

import org.ow2.easybeans.deployment.annotations.impl.JAnnotationResource;
import org.ow2.easybeans.deployment.annotations.impl.JEjbEJB;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceContext;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceUnit;
import org.ow2.easybeans.deployment.metadata.interfaces.ISharedMetadata;

/**
 * Defines Metadata shared by Field, Method and Classes. For example
 * &#64;javax.annotation.EJB, &#64;javax.annotation.Resource, etc.
 * @author Florent Benoit
 */
public class CommonMetadata implements ISharedMetadata {

    /**
     * This field is used as a PersistenceContext.
     */
    private JavaxPersistenceContext javaxPersistenceContext = null;

    /**
     * This field is used as a PersistenceUnit.
     */
    private JavaxPersistenceUnit javaxPersistenceUnit = null;

    /**
     * Object representing &#64;{@link javax.ejb.EJB} annotation.
     */
    private JEjbEJB jEjbEJB = null;

    /**
     * Object representing &#64;{@link javax.annotation.Resource} annotation.
     */
    private JAnnotationResource jAnnotationResource = null;

    /**
     * Constructor.
     */
    public CommonMetadata() {
    }

    /**
     * @return JEjbEJB object representing javax.ejb.EJB annotation.
     */
    public JEjbEJB getJEjbEJB() {
        return jEjbEJB;
    }

    /**
     * Set JAnnotationEJB object.
     * @param jEjbEJB object representing javax.annotation.EJB annotation.
     */
    public void setJEjbEJB(final JEjbEJB jEjbEJB) {
        this.jEjbEJB = jEjbEJB;
    }

    /**
     * @return JAnnotationResource object representing javax.annotation.Resource
     *         annotation.
     */
    public JAnnotationResource getJAnnotationResource() {
        return jAnnotationResource;
    }

    /**
     * Set JAnnotationResource object.
     * @param jAnnotationResource object representing javax.annotation.Resource
     *        annotation.
     */
    public void setJAnnotationResource(final JAnnotationResource jAnnotationResource) {
        this.jAnnotationResource = jAnnotationResource;
    }

    /**
     * @return true if this field is used as a persistence context.
     */
    public boolean isPersistenceContext() {
        return javaxPersistenceContext != null;
    }

    /**
     * @return the persistence context infos.
     */
    public JavaxPersistenceContext getJavaxPersistenceContext() {
        return javaxPersistenceContext;
    }

    /**
     * Sets the persistence context info on this field.
     * @param javaxPersistenceContext information on persistence context.
     */
    public void setJavaxPersistenceContext(final JavaxPersistenceContext javaxPersistenceContext) {
        this.javaxPersistenceContext = javaxPersistenceContext;
    }

    /**
     * @return true if this field is used as a persistence unit.
     */
    public boolean isPersistenceUnit() {
        return javaxPersistenceUnit != null;
    }

    /**
     * @return the persistence unit infos.
     */
    public JavaxPersistenceUnit getJavaxPersistenceUnit() {
        return javaxPersistenceUnit;
    }

    /**
     * Sets the persistence unit info on this field.
     * @param javaxPersistenceUnit information on persistence unit.
     */
    public void setJavaxPersistenceUnit(final JavaxPersistenceUnit javaxPersistenceUnit) {
        this.javaxPersistenceUnit = javaxPersistenceUnit;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        String titleIndent = " ";
        // classname
        sb.append(titleIndent);
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // jEjbEJB
        concatStringBuilder("jEjbEJB", jEjbEJB, sb);

        // jAnnotationResource
        concatStringBuilder("jAnnotationResource", jAnnotationResource, sb);

        // javaxPersistenceContext
        concatStringBuilder("javaxPersistenceContext", javaxPersistenceContext, sb);

        // javaxPersistenceUnit
        concatStringBuilder("javaxPersistenceUnit", javaxPersistenceUnit, sb);

        sb.append(titleIndent);
        sb.append("]\n");
        return sb.toString();
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     * @param indent the indent to add at each line.
     */
    public static void concatStringBuilder(final String name, final Object object, final StringBuilder sb,
            final String indent) {
        if (object instanceof Boolean) {
            // do not print if false.
            if (!((Boolean) object).booleanValue()) {
                return;
            }
        }
        if (object != null) {
            sb.append(indent);
            sb.append(name);
            sb.append("=");
            // For arrays, use asList
            if (object instanceof Object[]) {
                sb.append(Arrays.asList((Object[]) object));
            } else {
                sb.append(object);
            }
            sb.append("\n");
        }
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     */
    public static void concatStringBuilder(final String name, final Object object, final StringBuilder sb) {
        concatStringBuilder(name, object, sb, "    ");
    }

}
